/*
 * gtkpasman
 * Copyright (C) 2003 Cyril Bellot
 *
 * configread.c
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <gdk/gdkkeysyms.h>
#include <gtk/gtk.h>
#include <errno.h>
#include "config.h"

#define max_line_length 256

char *passwdfile;
char *sshcmd;
char *csshcmd;
char *ftpcmd;
char *telnetcmd;
char *httpcmd;

char *globalpasswd;
int decrypted_passwds;


/*
 * config_read
 * 	Read config from rc file
 */
int
config_read ()
{
  gchar buffer[max_line_length];
  FILE *fp;

  /* default values */
  passwdfile = g_strdup (DEFAULT_GPASSMAN_FILE);
  sshcmd = g_strdup (DEFAULT_SSHCMD);
  csshcmd = g_strdup (DEFAULT_CSSHCMD);
  ftpcmd = g_strdup (DEFAULT_FTPCMD);
  telnetcmd = g_strdup (DEFAULT_TELNETCMD);
  httpcmd = g_strdup (DEFAULT_HTTPCMD);

  /* start reading configuration file */
  if ((fp = fopen (GTKPASMAN_CONF_FILE, "rt")) == NULL) {
    g_free (passwdfile);
    passwdfile = g_strdup (DEFAULT_GPASSMAN_FILE);
    g_free (sshcmd);
    sshcmd = g_strdup (DEFAULT_SSHCMD);
    g_free (csshcmd);
    csshcmd = g_strdup (DEFAULT_CSSHCMD);
    g_free (ftpcmd);
    ftpcmd = g_strdup (DEFAULT_FTPCMD);
    g_free (telnetcmd);
    telnetcmd = g_strdup (DEFAULT_TELNETCMD);
    g_free (httpcmd);
    httpcmd = g_strdup (DEFAULT_HTTPCMD);
    return 0;
  }

  /* 1st line: passwd file */
  if (fgets (buffer, max_line_length, fp) != NULL) {
    g_free (passwdfile);
    passwdfile = g_strdup (g_strchomp (buffer));
  }

  /* 2nd line: ssh command */
  if (fgets (buffer, max_line_length, fp) != NULL) {
    g_free (sshcmd);
    sshcmd = g_strdup (g_strchomp (buffer));
  }

  /* 3rd line: ftp command */
  if (fgets (buffer, max_line_length, fp) != NULL) {
    g_free (ftpcmd);
    ftpcmd = g_strdup (g_strchomp (buffer));
  }

  /* 4th line: telnet command */
  if (fgets (buffer, max_line_length, fp) != NULL) {
    g_free (telnetcmd);
    telnetcmd = g_strdup (g_strchomp (buffer));
  }

  /* 5th line: cssh command */
  if (fgets (buffer, max_line_length, fp) != NULL) {
    g_free (csshcmd);
    csshcmd = g_strdup (g_strchomp (buffer));
  }

  /* 6th line: http command */
  if (fgets (buffer, max_line_length, fp) != NULL) {
    g_free (httpcmd);
    httpcmd = g_strdup (g_strchomp (buffer));
  }

  fclose (fp);
  return 1;
}

/*
 * config_write
 * 	write configuration file
 */
int
config_write ()
{
  FILE *fp;

  if ((fp = fopen (GTKPASMAN_CONF_FILE, "w+")) == NULL) {
    g_printerr ("Failed to open config: %s\n", strerror(errno));
    return 0;
  }
  /* write to file */
  if (fputs
      (g_strdup_printf
       ("%s\n%s\n%s\n%s\n%s\n%s", passwdfile, sshcmd,
	ftpcmd, telnetcmd, csshcmd, httpcmd), fp) == EOF) {
    g_printerr ("Failed to write config: %s\n", strerror(errno));
    return 0;
  }

  fclose (fp);
  return 1;
}
