/*
 * gtkpasman
 * Copyright (C) 2003 Cyril Bellot
 *
 * interface.c
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */
#include <sys/types.h>
#include <sys/stat.h>
#include <unistd.h>
#include <stdlib.h>
#include <string.h>
#include <stdio.h>
#include <gdk/gdkkeysyms.h>
#include <gtk/gtk.h>
#include "config.h"
#include "interface.h"
#include "passwd.h"
#include "configread.h"

void open_file (GtkFileSelection *FileSelect);
void select_file (void);
gboolean key_pressed (GtkWidget *widget, GdkEventKey *event, gpointer window);
static void change_pass (GtkWidget * passtxt);
void search_in_list (GtkWidget * passtxt);
static void select_row (GtkCTree * ctree, GtkCTreeNode * node);
static void select_command (gchar ** data);
static void select_passwd (gchar ** data);
static void select_passwd_and_exec_command (gchar ** data);
void exec_command (gchar ** data);
void exec_multiple (gchar * hostlist);
static void button_copy_cmd_line (void);
static void button_copy_passwd (void);
static void button_copy_passwd_and_execute (void);
static void button_execute (void);
static void create_gtkpasman_config (void);
static void change_config (GtkWidget * dialog);
static void tree_collapse (void);
static void tree_expand1 (void);
static void tree_expand2 (void);
static void tree_unselectall (void);


/* widgets to be used in several places */
GtkWidget *gtkpasman1;
GtkWidget *current;
GdkPixmap *on;
GdkBitmap *mask_on;
GdkPixmap *off;
GdkBitmap *mask_off;
GtkItemFactory *item_factory1;

GtkWidget *tree1;
GtkWidget *clipboard;
GtkCTreeNode *selectednode;
GtkWidget *entry4;
GtkWidget *entry5;
GtkWidget *entry6;
GtkWidget *entry7;
GtkWidget *entry8;
GtkWidget *entry9;
GtkWidget *button5;



/* menu for list window */
static GtkItemFactoryEntry Menu1[] = {
  {"/_File", NULL, NULL, 0, "<Branch>"},
  {"/File/_Open file", NULL, select_file, 0, "<Item>"},
  {"/File/_Settings", NULL, create_gtkpasman_config, 0, "<Item>"},
  {"/File/_Quit", NULL, gtk_main_quit, 0, "<Item>"},
  {"/Tree", NULL, NULL, 0, "<Branch>"},
  {"/Tree/Collapse all", NULL, tree_collapse, 0, "<Item>"},
  {"/Tree/Expand a little", NULL, tree_expand1, 0, "<Item>"},
  {"/Tree/Expand all", NULL, tree_expand2, 0, "<Item>"},
  {"/Tree/Unselect all", NULL, tree_unselectall, 0, "<Item>"},
  {"/Refresh", NULL, create_gtkpasman_askpassword, 0, "<Item>"},
};
#define Menu1nb	10

/*
 * open_file
 *      open a selected password file
 */
void
open_file (GtkFileSelection *FileSelect)
{
	g_free (passwdfile);
	passwdfile = g_strdup (gtk_file_selection_get_filename (FileSelect));
	config_write ();
	create_gtkpasman_askpassword ();
}

/*
 * select_file 
 *      select a password file
 */
void
select_file (void)
{
	GtkWidget *FileSelector;
	GtkWidget *Button;

	FileSelector = gtk_file_selection_new(
			"Select a new password file");

    /* ok_button */
    Button = GTK_FILE_SELECTION(FileSelector)->ok_button;
    gtk_signal_connect_object(GTK_OBJECT(Button), "clicked",
			(GtkSignalFunc)open_file,
			GTK_OBJECT(FileSelector));
	gtk_signal_connect_object(GTK_OBJECT(Button), "clicked",
			(GtkSignalFunc)gtk_widget_destroy,
		    GTK_OBJECT(FileSelector));

	/* Cancel_button */
	Button = GTK_FILE_SELECTION(FileSelector)->
			cancel_button;
    gtk_signal_connect_object(GTK_OBJECT(Button), "clicked",
			(GtkSignalFunc)gtk_widget_destroy,
			GTK_OBJECT(FileSelector));

	gtk_widget_show(FileSelector);
}


/*
 * create_gtkpasman_list
 *      create the list window and all the widgets
 */
GtkWidget *
create_gtkpasman_list (void)
{
  GtkWidget *vbox;
  GtkWidget *hbox1, *hbox2;
  GtkWidget *button;
  GtkWidget *menubar1;
  GtkWidget *scroll1;
  GtkWidget *label1;
  GtkWidget *searchtxt;
  GtkAccelGroup *menu1_accels;

  /* main window */
  gtkpasman1 = gtk_window_new (GTK_WINDOW_TOPLEVEL);
  gtk_window_set_title (GTK_WINDOW (gtkpasman1), "gtkpasman");
  gtk_window_set_wmclass (GTK_WINDOW (gtkpasman1),
			  "gtkpasmanMain", "gtkpasman");
  gtk_signal_connect (GTK_OBJECT (gtkpasman1), "delete_event",
		      (GtkSignalFunc) gtk_main_quit, NULL);
  gtk_signal_connect (GTK_OBJECT (gtkpasman1), "destroy_event",
		      (GtkSignalFunc) gtk_main_quit, NULL);
  vbox = gtk_vbox_new (FALSE, 0);
  gtk_container_add (GTK_CONTAINER (gtkpasman1), vbox);

  /* menus */
  menu1_accels = gtk_accel_group_new ();
  item_factory1 = gtk_item_factory_new (GTK_TYPE_MENU_BAR, "<Main>",
					menu1_accels);
  gtk_item_factory_create_items (item_factory1, Menu1nb, Menu1, NULL);
  //gtk_accel_group_attach (menu1_accels, GTK_OBJECT (gtkpasman1));
  menubar1 = gtk_item_factory_get_widget (item_factory1, "<Main>");
  gtk_box_pack_start (GTK_BOX (vbox), menubar1, FALSE, FALSE, 0);

  /* scrollwindow */
  scroll1 = gtk_scrolled_window_new (NULL, NULL);
  gtk_box_pack_start_defaults (GTK_BOX (vbox), scroll1);
  gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (scroll1),
				  GTK_POLICY_NEVER, GTK_POLICY_AUTOMATIC);

  /* tree */
  tree1 = gtk_ctree_new (1, 0);
  gtk_container_add (GTK_CONTAINER (scroll1), tree1);
  gtk_widget_set_usize (tree1, 50, 100);
  gtk_signal_connect (GTK_OBJECT (tree1), "tree_select_row",
		      (GtkSignalFunc) select_row, NULL);
  gtk_clist_set_selection_mode(GTK_CLIST(tree1), GTK_SELECTION_MULTIPLE);

  /* search entry */
  hbox1 = gtk_hbox_new (FALSE, 0);
  gtk_box_pack_start (GTK_BOX (vbox), hbox1, FALSE, FALSE, 0);

  label1 = gtk_label_new ("Search : ");
  gtk_label_set_justify (GTK_LABEL (label1), GTK_JUSTIFY_LEFT);
  gtk_box_pack_start (GTK_BOX (hbox1), label1, FALSE, FALSE, 0);
  searchtxt = gtk_entry_new ();
  gtk_box_pack_end (GTK_BOX (hbox1), searchtxt, TRUE, TRUE, 0);
  gtk_signal_connect_object (GTK_OBJECT (searchtxt), "activate",
			     (GtkSignalFunc) search_in_list,
			     GTK_OBJECT (searchtxt));

  /* bottom buttons */
  hbox2 = gtk_hbox_new (FALSE, 0);
  gtk_box_pack_start (GTK_BOX (vbox), hbox2, FALSE, FALSE, 0);
  gtk_container_set_border_width (GTK_CONTAINER (hbox2), 2);
  button = gtk_button_new_with_label ("Copy\ncommand\nline");
  gtk_box_pack_start (GTK_BOX (hbox2), button, TRUE, FALSE, 0);
  gtk_signal_connect (GTK_OBJECT (button), "clicked",
		      (GtkSignalFunc) button_copy_cmd_line, NULL);
  button = gtk_button_new_with_label ("Copy\npasswd");
  gtk_box_pack_start (GTK_BOX (hbox2), button, TRUE, FALSE, 0);
  gtk_signal_connect (GTK_OBJECT (button), "clicked",
		      (GtkSignalFunc) button_copy_passwd, NULL);
  button = gtk_button_new_with_label ("Copy\npasswd\n& execute");
  gtk_box_pack_start (GTK_BOX (hbox2), button, TRUE, FALSE, 0);
  gtk_signal_connect (GTK_OBJECT (button), "clicked",
		      (GtkSignalFunc) button_copy_passwd_and_execute, NULL);
  button = gtk_button_new_with_label ("Execute");
  gtk_box_pack_end (GTK_BOX (hbox2), button, TRUE, FALSE, 0);
  gtk_signal_connect (GTK_OBJECT (button), "clicked",
		      (GtkSignalFunc) button_execute, NULL);

  /* show all */
  gtk_widget_show_all (gtkpasman1);

  /* then add hidden entry used to have paste selections */
  clipboard = gtk_entry_new ();
  gtk_box_pack_start (GTK_BOX (vbox), clipboard, FALSE, TRUE, 0);
  gtk_widget_show (clipboard);
  gtk_widget_hide (clipboard);

  return gtkpasman1;
}

/*
 * create_gtkpasman_askpassword
 *      create the asking passphrase window
 */
void
create_gtkpasman_askpassword (void)
{
  GtkWidget *dialog;
  GtkWidget *passtxt;
  GtkWidget *okbutton;
  GtkWidget *cancelbutton;

  /* empty tree */
  emptytree (tree1);

  /* main windows (modal) */
  dialog = gtk_dialog_new ();
  gtk_window_set_position (GTK_WINDOW (dialog), GTK_WIN_POS_CENTER);
  gtk_window_set_modal (GTK_WINDOW (dialog), TRUE);
  gtk_window_set_title (GTK_WINDOW (dialog), "PGP Passphrase?");
  gtk_window_set_wmclass (GTK_WINDOW (dialog), "ReadPassword", "gtkpasman");
  //gtk_signal_connect (GTK_OBJECT (dialog), "key-press-event",
  //				 (GtkSignalFunc) gtk_widget_destroy,
  //				 GTK_OBJECT (dialog));


  /* password entry */
  passtxt = gtk_entry_new ();
  gtk_box_pack_start (GTK_BOX (GTK_DIALOG (dialog)->vbox),
		      passtxt, FALSE, FALSE, 0);
  gtk_entry_set_visibility (GTK_ENTRY (passtxt), FALSE);
  gtk_widget_grab_focus (passtxt);
  gtk_signal_connect_object (GTK_OBJECT (passtxt), "activate",
			     (GtkSignalFunc) change_pass,
			     GTK_OBJECT (passtxt));
  gtk_signal_connect_object (GTK_OBJECT (passtxt), "activate",
			     (GtkSignalFunc) gtk_widget_destroy,
			     GTK_OBJECT (dialog));

  /* cancel */
  cancelbutton = gtk_button_new_with_label ("Cancel");
  gtk_box_pack_start (GTK_BOX (GTK_DIALOG (dialog)->action_area),
		      cancelbutton, TRUE, FALSE, 0);
  gtk_signal_connect_object (GTK_OBJECT (cancelbutton), "clicked",
			     (GtkSignalFunc) gtk_widget_destroy,
			     GTK_OBJECT (dialog));

  /* ok */
  okbutton = gtk_button_new_with_label ("Ok");
  gtk_box_pack_start (GTK_BOX (GTK_DIALOG (dialog)->action_area),
		      okbutton, TRUE, FALSE, 0);
  gtk_signal_connect_object (GTK_OBJECT (okbutton), "clicked",
			     (GtkSignalFunc) change_pass,
			     GTK_OBJECT (passtxt));
  gtk_signal_connect_object (GTK_OBJECT (okbutton), "clicked",
			     (GtkSignalFunc) gtk_widget_destroy,
			     GTK_OBJECT (dialog));

  /* show all */
  gtk_widget_show_all (dialog);
}

/*
 * update_display
 *      Used to be: show or hide eye and list
 */
void
update_display (void)
{
    gtk_widget_show (gtkpasman1);
}

/*
 * change_pass
 *      change the password and try to fill the tree
 */
void
change_pass (GtkWidget * passtxt)
{
  /* read password from the entry widget */
  globalpasswd = g_strdup (gtk_entry_get_text (GTK_ENTRY (passtxt)));

  /* empty tree */
  emptytree (tree1);

  /* read file and fill tree */
  decrypted_passwds = filltree (tree1);

  /* No need to keep passphrase in memory now... */
  g_free (globalpasswd);
}

/*
 * compare
 *		find if a string b is in the 3rd data of a node pointed by a
 */
gint
compare (gconstpointer a, gconstpointer b)
{
	if (a) {
		gchar *a1, *a2;
		gint t;

		a1 = g_strdup((gchar *)(((gchar **)a)[2]));
		a2 = g_strdup((gchar *)(b));
		g_strdown(a1);
		g_strdown(a2);
		t=!strstr(a1, a2);
		g_free(a1);
		g_free(a2);
		return t;
	}
	return -1;
}

/*
 * search_in_list
 *      select the next label found in list
 */
void
search_in_list (GtkWidget * searchtxt)
{
  gchar *searchpat;
  GtkCTreeNode *i;

  /* read search pattern from the entry widget */
  searchpat = g_strdup (gtk_entry_get_text (GTK_ENTRY (searchtxt)));

  /* search in ctree from selectednode */
  selectednode = gtk_ctree_find_by_row_data_custom (GTK_CTREE(tree1),
		  selectednode,
		  searchpat, (GCompareFunc)compare);
  if (selectednode) {
	  /* select found node */
	  gtk_ctree_select (GTK_CTREE(tree1), selectednode);
	  /* expand childs */
	  gtk_ctree_expand_recursive (GTK_CTREE(tree1), selectednode);
	  /* expand parents up to root */
	  for (i = selectednode; i; i=GTK_CTREE_ROW(i)->parent) {
		  gtk_ctree_expand (GTK_CTREE(tree1), i);
	  }
	  /* scroll to this part of tree */
	  gtk_ctree_node_moveto (GTK_CTREE(tree1), selectednode, 0, 0.2, 0);
  }
  /* Free resources */
  g_free (searchpat);
}

/*
 * select_row
 *      keep track of the selected node
 */
void
select_row (GtkCTree * ctree, GtkCTreeNode * node)
{
  selectednode = node;
}

/*
 * select_command
 *      put command in the clipboard
 */
void
select_command (gchar ** data)
{
  gchar *txt;

  if (data != NULL) {
    txt = data[1];
    gtk_entry_set_text (GTK_ENTRY (clipboard), txt);
    gtk_editable_select_region (GTK_EDITABLE (clipboard), 0, -1);
    gtk_editable_copy_clipboard (GTK_EDITABLE (clipboard));
  }
}

/*
 * select_passwd
 *      put password in the clipboard
 */
void
select_passwd (gchar ** data)
{
  gchar *txt;

  if (data != NULL) {
    txt = data[0];
    gtk_entry_set_text (GTK_ENTRY (clipboard), txt);
    gtk_editable_select_region (GTK_EDITABLE (clipboard), 0, -1);
    gtk_editable_copy_clipboard (GTK_EDITABLE (clipboard));
  }
}

/*
 * select_passwd_and_exec_command
 *      put password in the clipboard
 *      execute command
 */
void
select_passwd_and_exec_command (gchar ** data)
{
  gchar *txt;

  select_passwd (data);
  if (data != NULL) {
    txt = data[1];
    system (txt);
  }
}

/*
 * exec_command
 *      execute command
 */
void
exec_command (gchar ** data)
{
  gchar *txt;

  if (data != NULL) {
    txt = data[1];
    system (txt);
  }
}

/*
 * exec_multiple
 *      execute command whith multiple selection
 */
void
exec_multiple (gchar * hostlist)
{
  gchar c = ' ', *s;
  gchar *cmd, *res, *tmp;

  cmd = g_strdup (csshcmd);
  
  /* replace %u by values */
  res = g_strdup ("");
  tmp = g_strdup ("");
  for (s = cmd; *s != '\0'; ++s) {
    if (*s == '%' && *(s + 1) != '\0') {
	  if ((c = *(s + 1)) == 'h') {
        tmp = g_strconcat (res, hostlist, NULL);
      }
	  else {
	    tmp = g_strdup_printf ("%s%c", res, *s);
	  }
      s++; 
    }
    else { 
      tmp = g_strdup_printf ("%s%c", res, *s);
    }
    g_free (res);
    res = g_strdup (tmp);
    g_free (tmp);
  } 

  /* execute command */
  system(res);

  /* free resources */
  g_free (cmd);
  g_free (res);
}

/*
 * button_copy_cmd_line
 *      callback for button
 */
void
button_copy_cmd_line (void)
{
  gchar **data;

  if (decrypted_passwds && selectednode) {
    data = (gchar **) gtk_ctree_node_get_row_data (GTK_CTREE (tree1),
						   selectednode);
    select_command (data);
  }
}

/*
 * button_copy_passwd
 *      callback for button
 */
void
button_copy_passwd (void)
{
  gchar **data;

  if (decrypted_passwds && selectednode) {
    data = (gchar **) gtk_ctree_node_get_row_data (GTK_CTREE (tree1),
						   selectednode);
    select_passwd (data);
  }
}

/*
 * button_copy_passwd_and_execute
 *      callback for button
 */
void
button_copy_passwd_and_execute (void)
{
  gchar **data;

  if (decrypted_passwds && selectednode) {
    data = (gchar **) gtk_ctree_node_get_row_data (GTK_CTREE (tree1),
						   selectednode);
    select_passwd_and_exec_command (data);
  }
}

/*
 * button_execute
 *      callback for button
 */
void
button_execute (void)
{
  gchar **data;
  gchar *hostlist;
  GList *i;
  int n = 0;

  hostlist = g_strdup("");
  for (i = GTK_CLIST(tree1)->selection; i; i=i->next) n++;
  if (n >1) {
	for (i = GTK_CLIST(tree1)->selection; i; i=i->next) {
	  data = (gchar **) gtk_ctree_node_get_row_data (GTK_CTREE (tree1), 
			  i->data);
	  if (data) hostlist = g_strdup_printf("%s %s",hostlist, data[3]);
	}
    exec_multiple(hostlist);
	g_free(hostlist);
  } else if (n == 1) {
	if (decrypted_passwds && selectednode) {
	  data = (gchar **) gtk_ctree_node_get_row_data (GTK_CTREE (tree1),
							 selectednode);
	  exec_command (data);
	}
  }
}

/*
 * menu_copy_passwd
 *      callback for menu
 */ 
void
menu_copy_passwd (gpointer data)
{   
  if (decrypted_passwds)
    select_passwd (data);
}
    
/*
 * menu_copy_passwd_and_execute
 *      callback for menu
 */ 
void
menu_copy_passwd_and_execute (gpointer data)
{
  if (decrypted_passwds)
    select_passwd_and_exec_command (data);
}

/*
 * strip_slashes_and_underscores
 *      strip slashes and underscores...
 */
gchar *
strip_slashes_and_underscores (gchar * str)
{

  g_print ("%s\n", str);

  /* backslash slashes */

  /* backslash underscores */

  /* Free resources */
  return str;
}

/*
 * create_gtkpasman_config
 *      create the configuration window
 */
void
create_gtkpasman_config (void)
{
  GtkWidget *dialog;
  GtkWidget *frame1;
  GtkWidget *frame2;
  GtkWidget *label4;
  GtkWidget *label5;
  GtkWidget *label6;
  GtkWidget *label7;
  GtkWidget *label8;
  GtkWidget *label9;
  GtkWidget *hbox2;
  GtkWidget *vbox2;
  GtkWidget *okbutton;
  GtkWidget *cancelbutton;

  /* main window (modal) */
  dialog = gtk_dialog_new ();
  gtk_window_set_position (GTK_WINDOW (dialog), GTK_WIN_POS_CENTER);
  gtk_window_set_modal (GTK_WINDOW (dialog), TRUE);
  gtk_window_set_title (GTK_WINDOW (dialog), "Setup");
  gtk_window_set_wmclass (GTK_WINDOW (dialog), "Setup", "gtkpasman");

  /* password file */
  frame1 = gtk_frame_new ("Password file");
  gtk_box_pack_start (GTK_BOX (GTK_DIALOG (dialog)->vbox),
		      frame1, FALSE, FALSE, 0);

  hbox2 = gtk_hbox_new (FALSE, 0);
  gtk_container_add (GTK_CONTAINER (frame1), hbox2);
  label4 = gtk_label_new ("Default file");
  gtk_box_pack_start (GTK_BOX (hbox2), label4, FALSE, FALSE, 0);
  entry4 = gtk_entry_new ();
  gtk_entry_set_text (GTK_ENTRY (entry4), passwdfile);
  gtk_box_pack_start (GTK_BOX (hbox2), entry4, FALSE, FALSE, 0);
  /* TODO: browse files */
  /* button = gtk_button_new_with_label ("Browse");
     gtk_box_pack_start (GTK_BOX(hbox2),
     button, FALSE, FALSE, 0); */

  /* Command lines */
  frame2 = gtk_frame_new ("Commands");
  gtk_box_pack_start (GTK_BOX (GTK_DIALOG (dialog)->vbox),
		      frame2, FALSE, FALSE, 0);

  vbox2 = gtk_vbox_new (FALSE, 0);
  gtk_container_add (GTK_CONTAINER (frame2), vbox2);

  label5 = gtk_label_new ("Ssh command:");
  gtk_box_pack_start (GTK_BOX (vbox2), label5, FALSE, FALSE, 0);
  entry5 = gtk_entry_new ();
  gtk_entry_set_text (GTK_ENTRY (entry5), sshcmd);
  gtk_box_pack_start (GTK_BOX (vbox2), entry5, FALSE, FALSE, 0);

  label9 = gtk_label_new ("Cssh command:");
  gtk_box_pack_start (GTK_BOX (vbox2), label9, FALSE, FALSE, 0);
  entry9 = gtk_entry_new ();
  gtk_entry_set_text (GTK_ENTRY (entry9), csshcmd);
  gtk_box_pack_start (GTK_BOX (vbox2), entry9, FALSE, FALSE, 0);

  label6 = gtk_label_new ("Ftp command:");
  gtk_box_pack_start (GTK_BOX (vbox2), label6, FALSE, FALSE, 0);
  entry6 = gtk_entry_new ();
  gtk_entry_set_text (GTK_ENTRY (entry6), ftpcmd);
  gtk_box_pack_start (GTK_BOX (vbox2), entry6, FALSE, FALSE, 0);

  label7 = gtk_label_new ("Telnet command:");
  gtk_box_pack_start (GTK_BOX (vbox2), label7, FALSE, FALSE, 0);
  entry7 = gtk_entry_new ();
  gtk_entry_set_text (GTK_ENTRY (entry7), telnetcmd);
  gtk_box_pack_start (GTK_BOX (vbox2), entry7, FALSE, FALSE, 0);

  label8 = gtk_label_new ("HTTP command:");
  gtk_box_pack_start (GTK_BOX (vbox2), label8, FALSE, FALSE, 0);
  entry8 = gtk_entry_new ();
  gtk_entry_set_text (GTK_ENTRY (entry8), httpcmd);
  gtk_box_pack_start (GTK_BOX (vbox2), entry8, FALSE, FALSE, 0);

  /* action area */
  cancelbutton = gtk_button_new_with_label ("Cancel");
  gtk_box_pack_start (GTK_BOX (GTK_DIALOG (dialog)->action_area),
		      cancelbutton, TRUE, FALSE, 0);
  gtk_signal_connect_object (GTK_OBJECT (cancelbutton), "clicked",
			     (GtkSignalFunc) gtk_widget_destroy,
			     GTK_OBJECT (dialog));

  okbutton = gtk_button_new_with_label ("Ok");
  gtk_box_pack_start (GTK_BOX (GTK_DIALOG (dialog)->action_area),
		      okbutton, TRUE, FALSE, 0);
  gtk_signal_connect_object (GTK_OBJECT (okbutton), "clicked",
			     (GtkSignalFunc) change_config,
			     GTK_OBJECT (dialog));

  /* show all */
  gtk_widget_show_all (dialog);
}

/*
 * change_config
 *      store the values from the configuration window
 */
void
change_config (GtkWidget * dialog)
{
  g_free (passwdfile);
  g_free (sshcmd);
  g_free (ftpcmd);
  g_free (telnetcmd);
  g_free (httpcmd);
  g_free (csshcmd);
  passwdfile = g_strdup (gtk_entry_get_text (GTK_ENTRY (entry4)));
  sshcmd = g_strdup (gtk_entry_get_text (GTK_ENTRY (entry5)));
  ftpcmd = g_strdup (gtk_entry_get_text (GTK_ENTRY (entry6)));
  telnetcmd = g_strdup (gtk_entry_get_text (GTK_ENTRY (entry7)));
  httpcmd = g_strdup (gtk_entry_get_text (GTK_ENTRY (entry8)));
  csshcmd = g_strdup (gtk_entry_get_text (GTK_ENTRY (entry9)));
  gtk_widget_destroy (dialog);
  update_display ();
  config_write ();
}

/*
 * tree_collapse
 * 		collapse all the nodes of the tree
 */
void
tree_collapse (void)
{
	gtk_ctree_collapse_recursive (GTK_CTREE(tree1), NULL);
}

/*
 * tree_expand1
 * 		expand all the nodes of the tree from root to level 2
 */
void
tree_expand1 (void)
{
	gtk_ctree_expand_to_depth (GTK_CTREE(tree1), NULL, 2);
}

/*
 * tree_expand2
 * 		expand all the nodes of the tree (5 levels max)
 */
void
tree_expand2 (void)
{
	gtk_ctree_expand_to_depth (GTK_CTREE(tree1), NULL, 5);
}

/*
 * tree_unselectall
 * 		Clear selection of the tree
 */
void
tree_unselectall (void)
{
	gtk_clist_unselect_all(GTK_CLIST(tree1));
}

